
#include "interrupt.h"
#include "stm32f4xx_tim.h"

extern xQueueHandle xQueueCAN2;



/**
  * @brief  Configures the NVIC.
  * @param  None
  * @retval None
  */
void InterruptCAN_Init(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;

  NVIC_InitStructure.NVIC_IRQChannel = CAN2_RX0_IRQn;

  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority =configMAX_SYSCALL_INTERRUPT_PRIORITY+1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

  NVIC_Init(&NVIC_InitStructure);
}

/**
  * @brief  ISR welche nach Empfang einer CAN Frame von CAN2 aufgerufen wird. Die ISR
  * 		Kopiert die CAN Frame in die Queue welche von dann von einer Task ausgelesen wird.
  * @param  None
  * @retval None
  */
void CAN2_RX0_IRQHandler(void)
{
  CanRxMsg RxMessage;
  portBASE_TYPE xHigherPriorityTaskWoken;

  RxMessage.StdId = 0x00;
  RxMessage.ExtId = 0x00;
  RxMessage.IDE = 0;
  RxMessage.DLC = 0;
  RxMessage.FMI = 0;
  RxMessage.Data[0] = 0x00;
  RxMessage.Data[1] = 0x00;

  xHigherPriorityTaskWoken = pdFALSE;

  CAN_Receive(CAN2, CAN_FIFO0, &RxMessage);

  xQueueSendToFrontFromISR( xQueueCAN2, &RxMessage, &xHigherPriorityTaskWoken );

  if( xHigherPriorityTaskWoken ==pdTRUE )
  	{
	  portYIELD();
  	}

}



