/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    31-October-2011
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/

#include <stdio.h>
#include "main.h"
#include"usercode.h"

/* Exported functions ---------------------------------------------------------*/

/* Global Variables ---------------------------------------------------------*/
xQueueHandle xQueueCAN2,xQueueLED,xQueueButton;

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

/*--------------- Tasks Priority -------------*/
#define REC_TASK_PRIO     ( tskIDLE_PRIORITY + 2 )
#define SEND_TASK_PRIO    ( tskIDLE_PRIORITY + 1 )
#define polynom 		  0x09

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
enum button	{
				 buttonpressed=0xA3,
				 buttonnotpressed=0x01,
			};

/* Private function prototypes -----------------------------------------------*/
void LED_Init(void);


/**
  * @brief  Diese Task nimmt von der ISR die CAN Frame entgegen und kopiert diese In die Queue f�r die
  * 		Aplication.
  * @param  pvParameters not used
  * @retval None
  */



/**
  * @brief  Main program anfangs werden alle Peripherien initialisiert
  * 		danach werden die Tasks erzeugt. Kurz bevor das Betriebssystem
  * 		gestartet wird, werden noch die CAN interrupts scharf geschaltet.
  * @param  None
  * @retval None
  */
int main(void)
{
	CanRxMsg RxMessage;
	portBASE_TYPE layer4Receiver = 0;
	portBASE_TYPE layer7Receiver = 0;
	portBASE_TYPE layer4Emitter = 0;
	portBASE_TYPE layer7Emitter = 0;
	int32_t checkTasks = 0;
	int32_t numTasks = 4;
	int32_t pauseValue;

	SystemInit();

	/*!< At this stage the microcontroller clock setting is already configured to
	   168 MHz, this is done through SystemInit() function which is called from
	   startup file (startup_stm32f4xx.s) before to branch to application main.
	   To reconfigure the default setting of SystemInit() function, refer to
	   system_stm32f4xx.c file
	 */

	/*Initialize Leds */
	LED_Init();

	/*Configure the Interrupt*/
	InterruptCAN_Init();

	CAN_Initialisierung();

	/* Create Tasks */
	xQueueCAN2 = xQueueCreate( 1, sizeof( RxMessage ) );
	xQueueLED = xQueueCreate( 1, sizeof( RxMessage ) );
	xQueueButton=xQueueCreate( 1, sizeof( RxMessage ) );

	if( xQueueCAN2 == 0 || xQueueLED ==0 || xQueueButton==0 )
	{
	  while(1);
	}


	//Handles received Messages From CAN2
	layer4Receiver = xTaskCreate(Layer4Receiver ,NULL, 512, NULL, REC_TASK_PRIO, NULL );

	//Gets the CAN Frame with the Status of the Button from Button Apli and sends the CAN Frame
	layer4Emitter = xTaskCreate(Layer4Emitter ,NULL, 512, NULL, REC_TASK_PRIO, NULL );

	//Checks the Status of the Button and puts the CAN Frame with the status into the Queue xQueueButton
	layer7Emitter = xTaskCreate( Layer7Emitter ,NULL, 512, NULL, REC_TASK_PRIO, NULL );

	//polls the user button and check the CAN status flags
	layer7Receiver = xTaskCreate( Layer7Receiver, NULL, 512, NULL, SEND_TASK_PRIO, NULL);

	checkTasks = (int32_t)layer4Receiver + (int32_t)layer4Emitter + (int32_t)layer7Receiver +
				 (int32_t)layer7Emitter;

	if(checkTasks != numTasks)
	{
		GPIO_SetBits(GPIOD, GPIO_Pin_15);
		GPIO_SetBits(GPIOD, GPIO_Pin_14);
		GPIO_SetBits(GPIOD, GPIO_Pin_13);
		GPIO_SetBits(GPIOD, GPIO_Pin_12);
		while(1)
		{
			for(pauseValue=0; pauseValue<1000000; pauseValue++);
			GPIO_ToggleBits(GPIOD, GPIO_Pin_15);
			GPIO_ToggleBits(GPIOD, GPIO_Pin_14);
			GPIO_ToggleBits(GPIOD, GPIO_Pin_13);
			GPIO_ToggleBits(GPIOD, GPIO_Pin_12);
		}
	}

	/*Enable CAN Interrupt*/
	CAN_ITConfig(CAN2, CAN_IT_FMP0, ENABLE);
	/*Enable CAN Interrupt*/
	CAN_ITConfig(CAN1, CAN_IT_FMP1, ENABLE);

	/* Start scheduler */
	vTaskStartScheduler();

	/* We should never get here as control is now taken by the scheduler */
	for( ;; );
}

/**
  * @brief  Initializes the LEDs resources and the input for the button
  * @param  None
  * @retval None
  */
void LED_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	/* Initialize Discovery's LEDs */
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

	/* Configure PD15 in output pushpull mode */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12|GPIO_Pin_13|  GPIO_Pin_14|  GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &GPIO_InitStructure);

	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);

	/* Configure PA0 input pulldown mode*/
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;

	GPIO_Init(GPIOA, &GPIO_InitStructure);

	GPIO_SetBits(GPIOD, GPIO_Pin_12);

}
/**
  * @brief  Calculates the CRC
  * @param  start pointer for the 7 bytes
  * @retval the CRC value
  */
uint8_t Calc_CRC(uint8_t *data)
{
	uint8_t i,j,nextbit,crcreg = 0;
	uint8_t shiftbyte = 0x80;

	for(j = 0;j < 7;j++)
	{

		for(i = 0;i < 8;i++)
			{
			shiftbyte=(0x80 >> i);

			nextbit=((crcreg & 0x80)^((data[j] & shiftbyte) << i));

			crcreg = crcreg << 1;

			if(nextbit)
				crcreg=crcreg^polynom;
			}
	}
	return crcreg;
}
/**
  * @brief  calculates the CRC (used for checking of received frames)
  * @param  the data to check
  * @retval the CRC value (0 if the received CRC is correct)
  */
uint8_t Check_REC_CRC(uint8_t *data)
{
	uint8_t i,j,nextbit,crcreg = 0;
	uint8_t shiftbyte = 0x80;

	for(j = 0;j < 8;j++)
	{

		for(i = 0;i < 8;i++)
			{
			shiftbyte=(0x80 >> i);

			nextbit=((crcreg & 0x80)^((data[j] & shiftbyte) << i));

			crcreg = crcreg << 1;

			if(nextbit)
				crcreg=crcreg^polynom;
			}
	}
	return crcreg;
}

/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
