#include"main.h"
#include"usercode.h"





//external functions and variables
extern xQueueHandle xQueueCAN2,xQueueLED,xQueueButton;
extern  uint8_t CAN_Send(CAN_TypeDef *CAN,CanTxMsg *Message);
//end of external usages


//prototypes
void ErrorState(void);
//end prototypes




/* Private functions ---------------------------------------------------------*/
/**
  * @brief  Application layer des Sensor Modules. Es �berpr�ft den Zustand des Tasters alle 100ms
  * 		und ca. alle 10s den Status des CAN Modules
  * @param  pvParameters not used
  * @retval None
  */
void Layer7Emitter(void * pvParameters)
{
	uint8_t i = 0;
	CanTxMsg Message;

	Message.StdId=0x012;

	Message.RTR = CAN_RTR_DATA;
	Message.IDE = CAN_ID_STD;
	Message.DLC = 8;

	for(i=0;i<8;i++)
		Message.Data[i] = 0x00;

	for( ;; )
	{
		while(1)
		{

			//----------------------------------------------
			if(GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0))
			{
				Message.Data[0]=0xA3;
				GPIO_SetBits(GPIOD, GPIO_Pin_15);

			}
			else
			{
				Message.Data[0]=0x00;
				GPIO_ResetBits(GPIOD, GPIO_Pin_15);
			}

			//-------------------------------------------------
			xQueueSendToFront( xQueueButton, &Message,0);

			vTaskDelay(100);

		}
	}
}



/**
* @brief  Diese Task erh�lt vom Application Layer des Sensors den Zustand des Tasters. Der Zustand wird alle
* 		20ms gesendet. Dazu wird die Queue vom Aplication Layer gepollt und wenn kein neuer Wert enthalten ist
* 		wird der alte Wert gesendet.
* @param  pvParameters not used
* @retval None
*/
void Layer4Emitter(void * pvParameters)
{
	uint8_t i;
    CanTxMsg Message;


	  Message.RTR = CAN_RTR_DATA;
	  Message.IDE = CAN_ID_STD;
	  Message.DLC = 8;

	  for(i=0;i<8;i++)
			Message.Data[i] = 0x00;

      for( ;; )
      {
    	xQueueReceive( xQueueButton, &( Message), 0 );		//get Button status
    	CAN_Send(CAN1,&Message);
        vTaskDelay(20);

      }
}


void Layer4Receiver ( void * pvParameters )
{
   CanRxMsg RxMessage;
   uint32_t tickCount		= 0;
   uint32_t oldTickCount	= 0;
   uint8_t  newDataIncomed	= 0;


    //xTaskGetTickCount(); returns actual Systick
    for(;;)
    {
    	newDataIncomed=xQueueReceive( xQueueCAN2, &( RxMessage ), 11 );
    	//hier code einfuegen


    	//ende der ergaenzung
    }

}


/**
  * @brief  Application Layer des Alarms. Wenn der Taster bet�tigt wurde wird die gr�ne LED eingeschaltet.
  * 		Diese Task wird ale 10ms gestartet und versucht eine CAN Frame aus der Queue zu holen, wenn die Queue
  * 		8 mal keine Frame enth�lt wird eine unterbrochene Verbindung angenommen und die LED (Alarm) eingeschaltet.
  * 		Zus�tzlich dazu wird mit der orangenen LED signalisiert, dass der deadcout abgelaufen ist bzw ein CRC-Fehler vorliegt.
  * @param  pvParameters not used
  * @retval None
  */
void Layer7Receiver( void * pvParameters )
{
	uint8_t buttonstate;
	uint8_t queue=0;
    CanRxMsg RxMessage;
	static portTickType xLastWakeTime;
	uint8_t layer4ReportedError = 0x01;
	xLastWakeTime = xTaskGetTickCount();


    for(;;)
    {

    	queue=xQueueReceive( xQueueLED, &( RxMessage ), 0 );
      	//------------------------------------------------------
    	if(queue==1)
    	{

    		buttonstate=RxMessage.Data[0];

    		if(buttonstate == 0xa3)
    		{
    			GPIO_ResetBits(GPIOD, GPIO_Pin_12);
    			GPIO_SetBits(GPIOD, GPIO_Pin_13);
    		}

    		if(buttonstate == 0x00)
    		{
    			GPIO_SetBits(GPIOD, GPIO_Pin_12);
    			GPIO_ResetBits(GPIOD, GPIO_Pin_13);
    		}

    		if(buttonstate == layer4ReportedError)
    		{
    			ErrorState();
    		}
    	}

      	vTaskDelayUntil(&xLastWakeTime, 53 );

    }

}

void ErrorState(void)
{
	GPIO_ResetBits(GPIOD, GPIO_Pin_12);
	GPIO_ResetBits(GPIOD, GPIO_Pin_13);
	GPIO_SetBits(GPIOD,GPIO_Pin_14);
	while(1)
	{
		vTaskDelay(10);
	}
}
